<?php

namespace RainLoop\Actions;

use RainLoop\Enumerations\Capa;
use RainLoop\Utils;

trait Attachments
{
	/**
	 * @throws \RuntimeException
	 */
	public function DoAttachmentsActions() : array
	{
		if (!$this->GetCapa(Capa::ATTACHMENTS_ACTIONS))
		{
			return $this->FalseResponse(__FUNCTION__);
		}

		$oAccount = $this->initMailClientConnection();

		$sAction = $this->GetActionParam('Do', '');
		$aHashes = $this->GetActionParam('Hashes', null);
		$aUids = $this->GetActionParam('Uids', null);
		$sFolder = $this->GetActionParam('Folder', null);

		$mResult = false;
		$bError = false;
		$aData = array();

		if (\is_array($aHashes) && \count($aHashes)) {
			foreach ($aHashes as $sZipHash) {
				$aResult = $this->getMimeFileByHash($oAccount, $sZipHash);
				if (!empty($aResult['FileHash'])) {
					$aData[] = $aResult;
				} else {
					$bError = true;
					break;
				}
			}
		} else if ($sFolder && \is_array($aUids) && \count($aUids)) {
			foreach ($aUids as $iUid) {
				// TODO: get all MimeIndex
/*
				$sMimeIndex = isset($aValues['MimeIndex']) ? (string) $aValues['MimeIndex'] : '';
				$aResult = $this->getMimeFilesByUid($oAccount, $sFolder, $iUid, $sMimeIndex);
				if (!empty($aResult['FileHash'])) {
					$aData[] = $aResult;
				} else {
					$bError = true;
					break;
				}
*/
			}
		}

		$oFilesProvider = $this->FilesProvider();
		if (!empty($sAction) && !$bError && \count($aData) && $oFilesProvider && $oFilesProvider->IsActive())
		{
			$bError = false;
			switch (\strtolower($sAction))
			{
				case 'zip':

					$sZipHash = \MailSo\Base\Utils::Sha1Rand();
					$sZipFileName = $oFilesProvider->GenerateLocalFullFileName($oAccount, $sZipHash);

					if (!empty($sZipFileName)) {
						if (\class_exists('ZipArchive')) {
							$oZip = new \ZipArchive();
							$oZip->open($sZipFileName, \ZIPARCHIVE::CREATE | \ZIPARCHIVE::OVERWRITE);
							$oZip->setArchiveComment('SnappyMail/'.APP_VERSION);
							foreach ($aData as $aItem) {
								$sFileName = (string) (isset($aItem['FileName']) ? $aItem['FileName'] : 'file.dat');
								$sFileHash = (string) (isset($aItem['FileHash']) ? $aItem['FileHash'] : '');
								if (!empty($sFileHash)) {
									$sFullFileNameHash = $oFilesProvider->GetFileName($oAccount, $sFileHash);
									if (!$oZip->addFile($sFullFileNameHash, $sFileName)) {
										$bError = true;
									}
								}
							}

							if (!$bError) {
								$bError = !$oZip->close();
							} else {
								$oZip->close();
							}
/*
						} else {
							@\unlink($sZipFileName);
							$oZip = new \SnappyMail\Stream\ZIP($sZipFileName);
//							$oZip->setArchiveComment('SnappyMail/'.APP_VERSION);
							foreach ($aData as $aItem) {
								$sFileName = (string) (isset($aItem['FileName']) ? $aItem['FileName'] : 'file.dat');
								$sFileHash = (string) (isset($aItem['FileHash']) ? $aItem['FileHash'] : '');
								if (!empty($sFileHash)) {
									$sFullFileNameHash = $oFilesProvider->GetFileName($oAccount, $sFileHash);
									if (!$oZip->addFile($sFullFileNameHash, $sFileName)) {
										$bError = true;
									}
								}
							}
							$oZip->close();
*/
						} else {
							@\unlink($sZipFileName);
							$oZip = new \PharData($sZipFileName . '.zip', 0, null, \Phar::ZIP);
							$oZip->compressFiles(\Phar::GZ);
							foreach ($aData as $aItem) {
								$sFileName = (isset($aItem['FileName']) ? (string) $aItem['FileName'] : 'file.dat');
								$sFileHash = (isset($aItem['FileHash']) ? (string) $aItem['FileHash'] : '');
								if ($sFileHash) {
									$sFullFileNameHash = $oFilesProvider->GetFileName($oAccount, $sFileHash);
									$oZip->addFile($sFullFileNameHash, $sFileName);
								}
							}
							$oZip->compressFiles(\Phar::GZ);
							unset($oZip);
							\rename($sZipFileName . '.zip', $sZipFileName);
						}

						foreach ($aData as $aItem) {
							$sFileHash = (isset($aItem['FileHash']) ? (string) $aItem['FileHash'] : '');
							if ($sFileHash) {
								$oFilesProvider->Clear($oAccount, $sFileHash);
							}
						}

						if (!$bError) {
							$mResult = array(
								'FileHash' => Utils::EncodeKeyValuesQ(array(
									'Account' => $oAccount ? $oAccount->Hash() : '',
									'FileName' => 'attachments.zip',
									'MimeType' => 'application/zip',
									'FileHash' => $sZipHash
								))
							);
						}
					}
					break;

				default:
					$data = new \SnappyMail\AttachmentsAction;
					$data->action = $sAction;
					$data->items = $aData;
					$data->filesProvider = $oFilesProvider;
					$data->account = $oAccount;
					$this->Plugins()->RunHook('json.attachments', array($data));
					$mResult = $data->result;
					break;
			}
		}
		else
		{
			$bError = true;
		}

		$this->requestSleep();
		return $this->DefaultResponse(__FUNCTION__, $bError ? false : $mResult);
	}

	private function getMimeFileByHash(\RainLoop\Model\Account $oAccount, string $sHash) : array
	{
		$aValues = $this->getDecodedRawKeyValue($sHash);

		$sFolder = isset($aValues['Folder']) ? (string) $aValues['Folder'] : '';
		$iUid = isset($aValues['Uid']) ? (int) $aValues['Uid'] : 0;
		$sMimeIndex = isset($aValues['MimeIndex']) ? (string) $aValues['MimeIndex'] : '';

		$sContentTypeIn = isset($aValues['MimeType']) ? (string) $aValues['MimeType'] : '';
		$sFileNameIn = isset($aValues['FileName']) ? (string) $aValues['FileName'] : '';

		$oFileProvider = $this->FilesProvider();

		$sResultHash = '';

		$mResult = $this->MailClient()->MessageMimeStream(function ($rResource, $sContentType, $sFileName, $sMimeIndex = '')
			use ($oAccount, $oFileProvider, $sFileNameIn, $sContentTypeIn, &$sResultHash) {

				unset($sContentType, $sFileName, $sMimeIndex);

				if (\is_resource($rResource))
				{
					$sHash = \MailSo\Base\Utils::Sha1Rand($sFileNameIn.'~'.$sContentTypeIn);
					$rTempResource = $oFileProvider->GetFile($oAccount, $sHash, 'wb+');

					if (\is_resource($rTempResource))
					{
						if (false !== \MailSo\Base\Utils::MultipleStreamWriter($rResource, array($rTempResource)))
						{
							$sResultHash = $sHash;
						}

						\fclose($rTempResource);
					}
				}

			}, $sFolder, $iUid, $sMimeIndex);

		$aValues['FileHash'] = $mResult ? $sResultHash : '';

		return $aValues;
	}

	private function getMimeFilesByUid(\RainLoop\Model\Account $oAccount, string $sFolder, int $iUid) : array
	{
/*
		$sMimeIndex = isset($aValues['MimeIndex']) ? (string) $aValues['MimeIndex'] : '';
		$sContentTypeIn = isset($aValues['MimeType']) ? (string) $aValues['MimeType'] : '';
		$sFileNameIn = isset($aValues['FileName']) ? (string) $aValues['FileName'] : '';
*/

		$oFileProvider = $this->FilesProvider();

		$sResultHash = '';

		$mResult = $this->MailClient()->MessageMimeStream(function ($rResource, $sContentType, $sFileName, $sMimeIndex = '')
			use ($oAccount, $iUid, $oFileProvider, &$sResultHash) {

				if (\is_resource($rResource))
				{
					$sHash = \MailSo\Base\Utils::Sha1Rand($iUid.'~'.$sFileName.'~'.$sContentType);
					$rTempResource = $oFileProvider->GetFile($oAccount, $sHash, 'wb+');

					if (\is_resource($rTempResource))
					{
						if (false !== \MailSo\Base\Utils::MultipleStreamWriter($rResource, array($rTempResource)))
						{
							$sResultHash = $sHash;
						}

						\fclose($rTempResource);
					}
				}

			}, $sFolder, $iUid, $sMimeIndex);

		return [
			'Folder' => $sFolder,
			'Uid' => $iUid,
//			'MimeIndex' => '',
			'MimeType' => '',
//			'FileName' => '',
			'FileHash' => $mResult ? $sResultHash : ''
		];
	}

}
